
public class tGramSchmidt {
	double[][] m;
	double[][] r;
	double[][] q;
	double[] y;
	int maxOrder;
	
	public tGramSchmidt(int size, double[][] matrix, double[] solution) {
		m = matrix;
		y = solution;
		maxOrder = size;
		r = new double[maxOrder][maxOrder];
		q = new double[maxOrder][maxOrder];
	}
	
	 double InnerProd(double[][] a, double[][] b, int colA, int colB, int order)
     {
         int i;
         double c = 0;
         for (i = 0; i < order; i++)
         {
             c = c + a[i][colA] * b[i][colB];
         }
         return c;
     }


     public double[] MultMatrixTransVect(double[][] tr, double[] b)
     {
         int j, k;
         double[] al = new double[b.length];
         double[] yl = new double[b.length];

         for (j = 0; j < b.length; j++)
         {
             al[j] = 0.0;
             for (k = 0; k < b.length; k++)
             {
                 al[j] = al[j] + (tr[k][j] * b[k]);
             }
         }
         return al;
     }


     double Len(double[] a)
     {
         int i;
         double len = 0.0;
         for (i = 0; i < a.length; i++)
         {
             len = len + a[i] * a[i];
         }
         len = Math.sqrt(len);
         return len;
     }

     public void CalcGranSchmidt()
     {
         int i, k;
         int actCol;
         double len = 0.0;
         double[] v = new double[maxOrder];
         for (i = 0; i < maxOrder; i++)
         {
             len = len + m[i][0] * m[i][0];
         }
         len = Math.sqrt(len);

         // q1 is different
         for (i = 0; i < maxOrder; i++)
         {
             q[i][0] = m[i][0] / len;
         }

         //r11 can be calculated right away
         r[0][0] = InnerProd(m, m, 0, 0, maxOrder)/len;

         for (actCol = 1; actCol < maxOrder; actCol++)
         {
             for (i = 0; i < maxOrder; i++)
             {
                 v[i] = m[i][actCol];
                 q[i][actCol] = 0.0;
             }

             for (i = 0; i < actCol; i++)
             {
                 // calculate the elements of R
                 r[i][actCol] = InnerProd(q, m, i, actCol, maxOrder);

                 // calculate the temporary V vector
                 for (k = 0; k < maxOrder; k++)
                 {
                     v[k] = v[k] - q[k][i] * r[i][actCol];
                 }
             }

             // the length of V 
             len = Len(v);

             // one column od Q is V / lenght of V
             for (i = 0; i < maxOrder; i++)
             {
                 q[i][actCol] = v[i] / len;
             }

             // the diagonal elements of R must be calculated separately
             r[actCol][actCol] = InnerProd(q, m, actCol, actCol, maxOrder);
         }
         y = MultMatrixTransVect(q, y);
     }


     public double[] Solve()
     {
         int k, l;
         double[] x = new double[maxOrder];
         for (k = maxOrder - 1; k >= 0; k--)
         {
             for (l = maxOrder - 1; l >= k; l--)
             {
                 y[k] = y[k] - x[l] * r[k][l];
             }
             x[k] = y[k] / r[k][k];
         }
         return x;
     }


}
